#ifndef _LIBCIRCUMFERENCE_IDMAP_H
#define _LIBCIRCUMFERENCE_IDMAP_H 1

#ifdef __cplusplus
#	include <cstdint>
#else
#	include <stdbool.h>
#	include <stdint.h>
#endif
#include <libHX/list.h>
#include <libHX/clist.h>
#include <libcircum/packet.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 * a3_idmap_cmd - command mapping
 * @reqrules_fixed:	list of request rules, fixed AVPs
 * @reqrules_var:	list of request rules, required/optional AVPs
 * @rsprules_fixed:	list of answer (response) rules, fixed AVPs
 * @rsprules_var:	list of answer rules, required/optional AVPs
 */
struct a3_idmap_cmd {
	uint32_t code;
	char *name;
	struct HXclist_head reqrules_fixed, reqrules_var,
		rsprules_fixed, rsprules_var;
};

/**
 * a3_idmap_avp - AVP mapping
 * @code:	Numeric code for AVP
 * @type:	Type of AVP
 * @name:	duh, name
 * @must:	AVP must have all the specified flags set (%A3_AVPFLAG_*)
 * @must_not:	AVP must not have the specified flags set (%A3_AVPFLAG_*)
 * @list:	chain of AVPs
 * @enum_list:	list of supported enum values
 * 		(if the AVP is of type %A3_AVPTYPE_ENUM)
 */
struct a3_idmap_avp {
	uint32_t code;
	enum a3_avp_type type;
	char *name;
	uint8_t must, must_not;
	struct HXlist_head list;
	union {
		struct HXbtree *enum_tree;
		struct {
			struct HXclist_head rules_fixed, rules_var;
		};
	};
};

struct a3_idmap_app {
	uint32_t id;
	char *name;
	struct HXlist_head list;
};

/* idmap.c */
extern unsigned int a3_idmap_app(const char *);
extern unsigned int a3_idmap_cmd(const char *);
extern unsigned int a3_idmap_avp(const char *);
extern unsigned int a3_idmap_enum(const char *, const char *);
extern bool a3_dict_register(const char *);

/* verify.c */
extern int a3_pkt_verify(struct a3_packet *);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* _LIBCIRCUMFERENCE_IDMAP_H */
