/*
 *	Copyright © Jan Engelhardt, 2008
 *
 *	This file is part of Codename Circumference. It is free software;
 *	you can redistribute it and/or modify it under the terms of the GNU
 *	Lesser General Public License as published by the Free Software
 *	Foundation, either version 3 of the License, or any later version.
 */
#include <pthread.h>
#include <signal.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <local/circum.h>
#include "internal.h"
#include "packet.h"

static pthread_mutex_t libcircum_init_lock = PTHREAD_MUTEX_INITIALIZER;
static bool libcircum_initialized;

/**
 * a3_initialize -
 *
 * Read command/AVP/application specifications and build the required data
 * structures. This must be called before being able to reasonably use
 * libcircum.
 */
EXPORT_SYMBOL bool a3_initialize(void)
{
	struct sigaction sa;
	char *s;
	/*
	 * Not using mutex_trylock() since we want to make sure every instance
	 * waits for completion of initialization.
	 */
	pthread_mutex_lock(&libcircum_init_lock);
	if (!libcircum_initialized) {
		if ((s = getenv("CIRCUM_DEBUG")) != NULL)
			circum_debug_flags = strtoul(s, NULL, 0);
		circum_net_init();
		circum_idmap_init();
		libcircum_initialized = true;
	}

	memset(&sa, 0, sizeof(sa));
	sa.sa_handler = SIG_IGN;
	sa.sa_flags   = SA_RESTART;
	if (sigaction(SIGPIPE, &sa, NULL) < 0)
		perror("sigaction SIGPIPE");
	pthread_mutex_unlock(&libcircum_init_lock);
	return true;
}

/**
 * a3_deinitialize - free all memory
 *
 * Mostly used for debugging memory leaks. Calling it from programs is not
 * really necessary when they exit anyway.
 */
EXPORT_SYMBOL void a3_deinitialize(void)
{
	circum_idmap_exit();
	circum_tls_exit();
}
