/*
 *	Minimal testsuite for libcircumplus
 *	Copyright © Jan Engelhardt, 2008
 *
 *	This file is part of Codename Circumference. It is free software;
 *	you can redistribute it and/or modify it under the terms of the GNU
 *	Lesser General Public License as published by the Free Software
 *	Foundation, either version 3 of the License, or any later version.
 */
#include <malloc.h>
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <libcircumplus.hpp>

static void pkt_construct_test(void)
{
	A3_packet *pkt;
	A3_avp *avp;
	unsigned int size;
	void *stream;

	pkt = new A3_packet();
	pkt->command  = a3_idmap_cmd("Capabilities-Exchange");
	pkt->cmdflags = A3_CMDFLAG_REQUEST;
	pkt->add_int("Vendor-Id", 1337);
	pkt->add_text("Product-Name", "Circumference Internal, libcircumplus");

	/* Nesting */
	avp = pkt->add_avp("Vendor-Specific-Application-Id", NULL, 0);
	avp->add_int("Auth-Application-Id", 0x4500);
	avp = avp->add_avp("Vendor-Specific-Application-Id", NULL, 0);
	avp->add_int("Auth-Application-Id", 0x4500);
	avp = avp->add_avp("Vendor-Specific-Application-Id", NULL, 0);
	avp = avp->add_avp("Vendor-Specific-Application-Id", NULL, 0);

/*
	avp = avp->add_avp(pkt, "UNREFERENCED GROUP", NULL, 0);
	avp->add_int("Auth-Application-Id", 0xffe);
	avp = avp->add_avp(pkt, "UNREFERENCED GROUP", NULL, 0);
	avp->add_int("Auth-Application-Id", 0xffd);
*/
	pkt->add_int("Auth-Application-Id", a3_idmap_app("WebAuth"));
	pkt->add_enum("Auth-Request-Type", "AUTHENTICATE_ONLY");

	pkt->treedump();

	stream = pkt->serialize(&size);
	if (stream == NULL)
		perror("royal failure in a3pkt_serialize");
	else
		circum_hexdump(stream, size);

	delete pkt;

	pkt = A3_packet::unserialize(stream, size);
	if (pkt == NULL)
		perror("royal failure in a3pkt_unserialize");
	else
		pkt->treedump();

	delete pkt;
	free(stream);
}

static void connection_test(const char *host)
{
	A3_conn conn;

	if (!conn.ctor_ok()) {
		perror("A3_conn()");
		return;
	}
	conn.appl_id() = a3_idmap_app("WebAuth");
	if (host == NULL)
		host = "2001:db8::34";
	if (conn.open(host))
		printf("New unit. Connected to %s.\n", host);
	else
		fprintf(stderr, "Connection to %s failed.\n", host);
}

int main(int argc, const char **argv)
{
	setvbuf(stdout, NULL, _IONBF, 0);
	setvbuf(stderr, NULL, _IONBF, 0);

	a3_initialize();
	pkt_construct_test();
	connection_test((argc >= 2) ? argv[1] : NULL);
	a3_deinitialize();
	return 0;
}
