/*
 *	Copyright © Jan Engelhardt, 2008
 *
 *	This file is part of Codename Circumference. It is free software;
 *	you can redistribute it and/or modify it under the terms of the GNU
 *	Lesser General Public License as published by the Free Software
 *	Foundation, either version 3 of the License, or any later version.
 */
#include <cstddef>
#include <libcircum/net.h>
#include "net.hpp"
#include "packet.hpp"

A3_conn::A3_conn(void)
{
	ll_conn = a3_conn_new();
}

bool A3_conn::ctor_ok(void) const
{
	return ll_conn != NULL;
}

bool A3_conn::open(const char *uri)
{
	return open_ok = a3_conn_open(ll_conn, uri);
}

A3_conn::~A3_conn(void)
{
	if (open_ok)
		a3_conn_close(ll_conn);
}

char *&A3_conn::session_id(void) const
{
	return ll_conn->session_id;
}

uint32_t &A3_conn::appl_id(void) const
{
	return ll_conn->appl_id;
}

A3_packet *A3_conn::recv(void)
{
	struct a3_packet *pkt = a3_conn_recv(ll_conn);
	if (pkt == NULL)
		return NULL;
	return new A3_packet(pkt);
}

ssize_t A3_conn::send(A3_packet *pkt)
{
	ssize_t ret = a3_conn_send(ll_conn, pkt->ll_pkt);
	pkt->ll_pkt = NULL;
	delete pkt;
	return ret;
}
