#ifndef _LIBCIRCUMFERENCEPLUS_PACKET_H
#define _LIBCIRCUMFERENCEPLUS_PACKET_H 1

#include <list>
#include <libHX/clist.h>
#include <libcircum/packet.h>

struct a3_idmap;

/**
 * A3_avp - Class-based wrapper for a3_avp
 */
class A3_avp {
	friend class A3_packet;

public:
	A3_avp *add_avp(const char *, const void *, unsigned int);
	A3_avp *add_int(const char *, long long);
	A3_avp *add_enum(const char *, const char *);
	A3_avp *add_text(const char *, const char *);
	A3_avp *find(const char *) const;

	std::list<A3_avp *> avp_list;

private:
	static struct a3_avp *alloc_ll(struct a3_packet *,
		const char *, const void *, unsigned int);
	static struct a3_avp *alloc_ll(struct a3_avp *,
		const char *, const void *, unsigned int);
	template<class ll_object> A3_avp(ll_object *,
		const char *, const void *, unsigned int);
	A3_avp(struct a3_avp *);
	~A3_avp(void);

	struct a3_avp *ll_avp;

public:
	uint32_t &code;
	uint8_t &flags;
	enum a3_avp_type &type;
	uint32_t &vendor_id;

	union a3_avp_morph &d;
	unsigned int &blob_length;
	char (&blob)[0];
};

class A3_packet {
	friend class A3_conn;

public:
	A3_packet(void);
	~A3_packet(void);

	A3_avp *add_avp(const char *, const void *, unsigned int);
	A3_avp *add_int(const char *, long long);
	A3_avp *add_enum(const char *, const char *);
	A3_avp *add_text(const char *, const char *);
	A3_avp *find(const char *) const;
	void treedump(void) const;
	void *serialize(unsigned int *) const;
	static A3_packet *unserialize(const void *, unsigned int);

private:
	static struct a3_packet *alloc_ll(void);
	A3_packet(struct a3_packet *);
	struct a3_packet *ll_pkt;

public:
	uint8_t &cmdflags;
	uint32_t &command;
	uint32_t &appl_id;
	uint32_t &hbh_id;
	uint32_t &ete_id;

	std::list<A3_avp *> avp_list;
};

#endif /* _LIBCIRCUMFERENCEPLUS_PACKET_H */
